@@ -152,7 +152,10 @@ where
 #[cfg(test)]
 mod test {
 	use super::LeafPrivate;
-	use crate::setup::mixer::*;
+	use crate::setup::{
+		common::{verify_unchecked_raw, *},
+		mixer::*,
+	};
 	use ark_bn254::{Bn254, Fr as Bn254Fr};
 	use ark_ff::{BigInteger, PrimeField, UniformRand};
 	use ark_groth16::{Groth16, Proof, VerifyingKey};
@@ -173,12 +176,11 @@ mod test {
 		let params3 = setup_params_x5_3::<Bn254Fr>(curve);
 		let params5 = setup_params_x5_5::<Bn254Fr>(curve);
 		let prover = MixerProverSetupBn254_30::new(params3, params5);
-		let (circuit, .., public_inputs) = prover.setup_random_circuit(rng);
+		let (circuit, .., public_inputs) = prover.setup_random_circuit(rng).unwrap();
 
-		let (pk, vk) = MixerProverSetupBn254_30::setup_keys::<Bn254, _>(circuit.clone(), rng);
-		let proof = MixerProverSetupBn254_30::prove::<Bn254, _>(circuit, &pk, rng);
-		let res = MixerProverSetupBn254_30::verify::<Bn254>(&public_inputs, &vk, &proof);
-		println!("{}", res);
+		let (pk, vk) = setup_keys::<Bn254, _, _>(circuit.clone(), rng).unwrap();
+		let proof = prove::<Bn254, _, _>(circuit, &pk, rng).unwrap();
+		let res = verify::<Bn254>(&public_inputs, &vk, &proof).unwrap();
 		assert!(res);
 	}
 
@@ -187,8 +189,6 @@ mod test {
 		let rng = &mut test_rng();
 		let curve = Curve::Bn254;
 
-		let leaves = vec![Bn254Fr::one()];
-		let index = 0;
 		let recipient = Bn254Fr::one();
 		let relayer = Bn254Fr::zero();
 		let fee = Bn254Fr::zero();
@@ -198,12 +198,20 @@ mod test {
 		let params5 = setup_params_x5_5::<Bn254Fr>(curve);
 		let prover = MixerProverSetupBn254_30::new(params3, params5);
 
-		let (circuit, .., public_inputs) =
-			prover.setup_circuit(&leaves, index, recipient, relayer, fee, refund, rng);
+		let (leaf_privates, leaf_hash, ..) = prover.setup_leaf(rng).unwrap();
+		let secret = leaf_privates.secret();
+		let nullifier = leaf_privates.nullifier();
+		let leaves = vec![leaf_hash];
+		let index = 0;
+		let (circuit, .., public_inputs) = prover
+			.setup_circuit_with_privates(
+				secret, nullifier, &leaves, index, recipient, relayer, fee, refund,
+			)
+			.unwrap();
 
-		let (pk, vk) = MixerProverSetupBn254_30::setup_keys::<Bn254, _>(circuit.clone(), rng);
-		let proof = MixerProverSetupBn254_30::prove::<Bn254, _>(circuit, &pk, rng);
-		let res = MixerProverSetupBn254_30::verify::<Bn254>(&public_inputs, &vk, &proof);
+		let (pk, vk) = setup_keys::<Bn254, _, _>(circuit.clone(), rng).unwrap();
+		let proof = prove::<Bn254, _, _>(circuit, &pk, rng).unwrap();
+		let res = verify::<Bn254>(&public_inputs, &vk, &proof).unwrap();
 		assert!(
 			res,
 			"Failed to verify  Proof, here is the inputs:
@@ -223,8 +231,6 @@ mod test {
 		let rng = &mut test_rng();
 		let curve = Curve::Bn254;
 
-		let leaves = vec![Bn254Fr::one()];
-		let index = 0;
 		let recipient = Bn254Fr::one();
 		let relayer = Bn254Fr::zero();
 		let fee = Bn254Fr::zero();
@@ -234,11 +240,19 @@ mod test {
 		let params5 = setup_params_x5_5::<Bn254Fr>(curve);
 		let prover = MixerProverSetupBn254_30::new(params3, params5);
 
-		let (circuit, .., public_inputs) =
-			prover.setup_circuit(&leaves, index, recipient, relayer, fee, refund, rng);
+		let (leaf_privates, leaf_hash, ..) = prover.setup_leaf(rng).unwrap();
+		let secret = leaf_privates.secret();
+		let nullifier = leaf_privates.nullifier();
+		let leaves = vec![leaf_hash];
+		let index = 0;
+		let (circuit, .., public_inputs) = prover
+			.setup_circuit_with_privates(
+				secret, nullifier, &leaves, index, recipient, relayer, fee, refund,
+			)
+			.unwrap();
 
-		let (pk, vk) = MixerProverSetupBn254_30::setup_keys::<Bn254, _>(circuit.clone(), rng);
-		let proof = MixerProverSetupBn254_30::prove::<Bn254, _>(circuit, &pk, rng);
+		let (pk, vk) = setup_keys::<Bn254, _, _>(circuit.clone(), rng).unwrap();
+		let proof = prove::<Bn254, _, _>(circuit, &pk, rng).unwrap();
 
 		let vk = VerifyingKey::<Bn254>::deserialize(&vk[..]).unwrap();
 		let proof = Proof::<Bn254>::deserialize(&proof[..]).unwrap();
@@ -261,11 +275,11 @@ mod test {
 		let params3 = setup_params_x5_3::<Bn254Fr>(curve);
 		let params5 = setup_params_x5_5::<Bn254Fr>(curve);
 		let prover = MixerProverSetupBn254_30::new(params3, params5);
-		let (leaf_private, leaf, nullifier_hash) = prover.setup_leaf(rng);
+		let (leaf_private, leaf, nullifier_hash) = prover.setup_leaf(rng).unwrap();
 
 		let arbitrary_input =
 			MixerProverSetupBn254_30::setup_arbitrary_data(recipient, relayer, fee, refund);
-		let (_, path) = prover.setup_tree_and_create_path(&[leaf], 0);
+		let (_, path) = prover.setup_tree_and_create_path(&[leaf], 0).unwrap();
 		let root = Bn254Fr::rand(rng);
 
 		let circuit = prover.create_circuit(
@@ -284,9 +298,9 @@ mod test {
 		public_inputs.push(arbitrary_input.fee);
 		public_inputs.push(arbitrary_input.refund);
 
-		let (pk, vk) = MixerProverSetupBn254_30::setup_keys::<Bn254, _>(circuit.clone(), rng);
-		let proof = MixerProverSetupBn254_30::prove::<Bn254, _>(circuit, &pk, rng);
-		let res = MixerProverSetupBn254_30::verify::<Bn254>(&public_inputs, &vk, &proof);
+		let (pk, vk) = setup_keys::<Bn254, _, _>(circuit.clone(), rng).unwrap();
+		let proof = prove::<Bn254, _, _>(circuit, &pk, rng).unwrap();
+		let res = verify::<Bn254>(&public_inputs, &vk, &proof).unwrap();
 
 		assert!(!res);
 	}
@@ -304,12 +318,12 @@ mod test {
 		let params3 = setup_params_x5_3::<Bn254Fr>(curve);
 		let params5 = setup_params_x5_5::<Bn254Fr>(curve);
 		let prover = MixerProverSetupBn254_30::new(params3, params5);
-		let (leaf_private, _, nullifier_hash) = prover.setup_leaf(rng);
+		let (leaf_private, _, nullifier_hash) = prover.setup_leaf(rng).unwrap();
 		let leaf = Bn254Fr::rand(rng);
 
 		let arbitrary_input =
 			MixerProverSetupBn254_30::setup_arbitrary_data(recipient, relayer, fee, refund);
-		let (tree, path) = prover.setup_tree_and_create_path(&[leaf], 0);
+		let (tree, path) = prover.setup_tree_and_create_path(&[leaf], 0).unwrap();
 		let root = tree.root().inner();
 
 		let circuit = prover.create_circuit(
@@ -328,9 +342,9 @@ mod test {
 		public_inputs.push(arbitrary_input.fee);
 		public_inputs.push(arbitrary_input.refund);
 
-		let (pk, vk) = MixerProverSetupBn254_30::setup_keys::<Bn254, _>(circuit.clone(), rng);
-		let proof = MixerProverSetupBn254_30::prove::<Bn254, _>(circuit, &pk, rng);
-		let res = MixerProverSetupBn254_30::verify::<Bn254>(&public_inputs, &vk, &proof);
+		let (pk, vk) = setup_keys::<Bn254, _, _>(circuit.clone(), rng).unwrap();
+		let proof = prove::<Bn254, _, _>(circuit, &pk, rng).unwrap();
+		let res = verify::<Bn254>(&public_inputs, &vk, &proof).unwrap();
 
 		assert!(!res);
 	}
@@ -348,14 +362,14 @@ mod test {
 		let params3 = setup_params_x5_3::<Bn254Fr>(curve);
 		let params5 = setup_params_x5_5::<Bn254Fr>(curve);
 		let prover = MixerProverSetupBn254_30::new(params3, params5);
-		let (leaf_private, leaf, nullifier_hash) = prover.setup_leaf(rng);
+		let (leaf_private, leaf, nullifier_hash) = prover.setup_leaf(rng).unwrap();
 
 		// Invalid nullifier
 		let leaf_private = LeafPrivate::new(leaf_private.secret(), Bn254Fr::rand(rng));
 
 		let arbitrary_input =
 			MixerProverSetupBn254_30::setup_arbitrary_data(recipient, relayer, fee, refund);
-		let (tree, path) = prover.setup_tree_and_create_path(&[leaf], 0);
+		let (tree, path) = prover.setup_tree_and_create_path(&[leaf], 0).unwrap();
 		let root = tree.root().inner();
 
 		let circuit = prover.create_circuit(
@@ -374,9 +388,9 @@ mod test {
 		public_inputs.push(arbitrary_input.fee);
 		public_inputs.push(arbitrary_input.refund);
 
-		let (pk, vk) = MixerProverSetupBn254_30::setup_keys::<Bn254, _>(circuit.clone(), rng);
-		let proof = MixerProverSetupBn254_30::prove::<Bn254, _>(circuit, &pk, rng);
-		let res = MixerProverSetupBn254_30::verify::<Bn254>(&public_inputs, &vk, &proof);
+		let (pk, vk) = setup_keys::<Bn254, _, _>(circuit.clone(), rng).unwrap();
+		let proof = prove::<Bn254, _, _>(circuit, &pk, rng).unwrap();
+		let res = verify::<Bn254>(&public_inputs, &vk, &proof).unwrap();
 
 		assert!(!res);
 	}
@@ -386,44 +400,48 @@ mod test {
 		let rng = &mut test_rng();
 		let curve = Curve::Bn254;
 
-		let leaves = vec![Bn254Fr::one()];
-		let index = 0;
 		let recipient = Bn254Fr::one();
 		let relayer = Bn254Fr::zero();
-		let secret = Bn254Fr::rand(rng);
-		let nullifier = Bn254Fr::rand(rng);
-
-		let leaves_raw: Vec<Vec<u8>> = leaves.iter().map(|x| x.into_repr().to_bytes_le()).collect();
-		let recipient_raw = recipient.into_repr().to_bytes_le();
-		let relayer_raw = relayer.into_repr().to_bytes_le();
 		let fee = 0;
 		let refund = 0;
-		let secret_raw = secret.into_repr().to_bytes_le();
-		let nullifier_raw = nullifier.into_repr().to_bytes_le();
+
+		let recipient_raw = recipient.into_repr().to_bytes_le();
+		let relayer_raw = relayer.into_repr().to_bytes_le();
 
 		let params3 = setup_params_x5_3::<Bn254Fr>(curve);
 		let params5 = setup_params_x5_5::<Bn254Fr>(curve);
 		let prover = MixerProverSetupBn254_30::new(params3, params5);
 
-		let (circuit, .., public_inputs_raw) = prover.setup_circuit_with_privates_raw(
-			secret_raw,
-			nullifier_raw,
-			&leaves_raw,
-			index,
-			recipient_raw,
-			relayer_raw,
-			fee,
-			refund,
-		);
+		let (leaf_privates, leaf_hash, ..) = prover.setup_leaf(rng).unwrap();
+		let secret = leaf_privates.secret();
+		let nullifier = leaf_privates.nullifier();
+		let index = 0;
+
+		let secret_raw = secret.into_repr().to_bytes_le();
+		let nullifier_raw = nullifier.into_repr().to_bytes_le();
+		let leaves_raw = vec![leaf_hash.into_repr().to_bytes_le()];
+
+		let (circuit, .., public_inputs_raw) = prover
+			.setup_circuit_with_privates_raw(
+				secret_raw,
+				nullifier_raw,
+				&leaves_raw,
+				index,
+				recipient_raw,
+				relayer_raw,
+				fee,
+				refund,
+			)
+			.unwrap();
 
 		let public_inputs: Vec<Bn254Fr> = public_inputs_raw
 			.iter()
 			.map(|x| Bn254Fr::from_le_bytes_mod_order(x))
 			.collect();
 
-		let (pk, vk) = MixerProverSetupBn254_30::setup_keys::<Bn254, _>(circuit.clone(), rng);
-		let proof = MixerProverSetupBn254_30::prove::<Bn254, _>(circuit, &pk, rng);
-		let res = MixerProverSetupBn254_30::verify::<Bn254>(&public_inputs, &vk, &proof);
+		let (pk, vk) = setup_keys::<Bn254, _, _>(circuit.clone(), rng).unwrap();
+		let proof = prove::<Bn254, _, _>(circuit, &pk, rng).unwrap();
+		let res = verify::<Bn254>(&public_inputs, &vk, &proof).unwrap();
 		assert!(
 			res,
 			"Failed to verify Proof, here is the inputs:
@@ -443,45 +461,43 @@ mod test {
 		let rng = &mut test_rng();
 		let curve = Curve::Bn254;
 
-		let leaves = vec![Bn254Fr::one()];
-		let index = 0;
 		let recipient = Bn254Fr::one();
 		let relayer = Bn254Fr::zero();
-		let secret = Bn254Fr::rand(rng);
-		let nullifier = Bn254Fr::rand(rng);
-
-		let leaves_raw: Vec<Vec<u8>> = leaves.iter().map(|x| x.into_repr().to_bytes_le()).collect();
-		let recipient_raw = recipient.into_repr().to_bytes_le();
-		let relayer_raw = relayer.into_repr().to_bytes_le();
 		let fee = 0;
 		let refund = 0;
-		let secret_raw = secret.into_repr().to_bytes_le();
-		let nullifier_raw = nullifier.into_repr().to_bytes_le();
+
+		let recipient_raw = recipient.into_repr().to_bytes_le();
+		let relayer_raw = relayer.into_repr().to_bytes_le();
 
 		let params3 = setup_params_x5_3::<Bn254Fr>(curve);
 		let params5 = setup_params_x5_5::<Bn254Fr>(curve);
 		let prover = MixerProverSetupBn254_30::new(params3, params5);
 
-		let (circuit, .., public_inputs_raw) = prover.setup_circuit_with_privates_raw(
-			secret_raw,
-			nullifier_raw,
-			&leaves_raw,
-			index,
-			recipient_raw,
-			relayer_raw,
-			fee,
-			refund,
-		);
+		let (leaf_privates, leaf_hash, ..) = prover.setup_leaf(rng).unwrap();
+		let secret = leaf_privates.secret();
+		let nullifier = leaf_privates.nullifier();
+		let index = 0;
 
-		let public_inputs: Vec<Bn254Fr> = public_inputs_raw
-			.iter()
-			.map(|x| Bn254Fr::from_le_bytes_mod_order(x))
-			.collect();
+		let secret_raw = secret.into_repr().to_bytes_le();
+		let nullifier_raw = nullifier.into_repr().to_bytes_le();
+		let leaves_raw = vec![leaf_hash.into_repr().to_bytes_le()];
 
-		let (pk, vk) =
-			MixerProverSetupBn254_30::setup_keys_unchecked::<Bn254, _>(circuit.clone(), rng);
-		let proof = MixerProverSetupBn254_30::prove_unchecked::<Bn254, _>(circuit, &pk, rng);
-		let res = MixerProverSetupBn254_30::verify_unchecked::<Bn254>(&public_inputs, &vk, &proof);
+		let (circuit, .., public_inputs_raw) = prover
+			.setup_circuit_with_privates_raw(
+				secret_raw,
+				nullifier_raw,
+				&leaves_raw,
+				index,
+				recipient_raw,
+				relayer_raw,
+				fee,
+				refund,
+			)
+			.unwrap();
+
+		let (pk, vk) = setup_keys_unchecked::<Bn254, _, _>(circuit.clone(), rng).unwrap();
+		let proof = prove_unchecked::<Bn254, _, _>(circuit, &pk, rng).unwrap();
+		let res = verify_unchecked_raw::<Bn254>(&public_inputs_raw, &vk, &proof).unwrap();
 		assert!(
 			res,
 			"Failed to verify Proof, here is the inputs:
@@ -492,7 +508,7 @@ mod test {
 			public_inputs = {:?},
 			proof = {:?},
 			",
-			recipient, relayer, fee, refund, public_inputs, proof
+			recipient, relayer, fee, refund, public_inputs_raw, proof
 		);
 	}
 }
